"""
Tamr - Golden Records
See https://docs.tamr.com/docs/overview-golden-records
"""
from tamr_client.golden_records import project
from tamr_client.golden_records._golden_records import (
    _publish_async,
    _update_async,
    publish,
    update,
)
